/** \file dlist.h
*/
#ifndef _D_MOD_DLIST


#define _D_MOD_DLIST

#define _D_VERSION_DLIST	"1.2.1"

#define	ERR_DLIST_OK			0
#define	ERR_DLIST_NO_VALID		-101
#define	ERR_DLIST_NO_MEM		-102
#define	ERR_DLIST_NO_COMPARE	-103
#define	ERR_DLIST_NO_ELEMENT	-104	

#define	DLIST_TYPE_DEF		0	
#define	DLIST_TYPE_DOUBLE	1	
#define	DLIST_TYPE_ARRAY	2	

typedef struct dlist_headv2	*dlist;

/** Creates a new empty list with "free" as element destroyer and no comparer function */
dlist dlist_new_list(void);

/** Creates a new empty list from an specific type, using "free" as the destroyer and no comparer function */
dlist dlist_new_list_bytype(int type);

/** Destroys a list */
void dlist_delete_list(dlist lst);

/** Destroys a list but it does not destroy the data in each list member */
void dlist_delete_list_butnotdata(dlist lst);

/** Set the function that will be used in order to free the memory used by any list element */
int dlist_set_destroyer(dlist lst, void (*destroyer)(void*));

/** Adds a new element in the first position of the list */
int dlist_add_first_element(dlist lst, void *element);

/** Adds a new element to a list */
int dlist_add_element(dlist lst, void *element);

/** Adds a new element in the last position of the list */
int dlist_add_last_element(dlist lst, void *element);

/** Returns the data in the first element of the list and removes it from the list */
void *dlist_get_first_element(dlist lst);

/**
 * Returns the data in the first element but does not remove it (nor makes a copy of the data).
 * If the parameter lst is NULL then it returns the next element in the same list.
 * Can not be used to scan two lists simultaneously. */
void *dlist_see_element(dlist lst);

/** 
 * Sets the function used to compare two list elements.
 * The function should return a value greater than 0 if the first argumento is greater than the second one. This way, the list could be order from greater to lesser.
 * If the function returns a value lesser than 0 when the first element is smaller than the second one then the list will keep an order from lesser to greater.
 * The function should return 0 if both element are equal. This will be use in order to check whether an element is or not in a list (see dlist_element_inlist())
 */
int dlist_set_compare(dlist lst, int (*compare)(void *, void*));

/** Adds a new element keeping the order in the list imposed by the comparer function */
int dlist_add_element_inorder(dlist lst, void *element);

/**
 * Removes an element from a list.
 * The element is not modified.
 */
int dlist_remove_element(dlist lst, const void *element);

/**
 * Returns the element from the list that is in "pos" position.
 * The positions are numbered starting by 1.
 * It also removes the element from the list.
 */
void *dlist_get_element_byposition(dlist lst, int pos);

/**
 * Returns the element from the list that is in "pos" position.
 * The positions are numbered starting by 1.
 * It does not remove the element from the list.
 */
void *dlist_see_element_byposition(dlist lst, int pos);

/** Returns the number of elements in the list */
int dlist_count_elements(dlist lst);

/**
 * Adds a new element to an ordered list.
 * It starts the search from the end of the list.
 * The list must be of type DLIST_TYPE_DOUBLE.
 */
int dlist_add_element_inorder_fromtheend(dlist lst, void *element);

/**
 * Check whether an element is in a list or not.
 * Returns 1 if the element is in the list and 0 if it is not.
 * Returns 0 in case of error.
 * It uses the "compare" function in order to decide whether the argument is equal or not to a list element. */
int dlist_element_inlist(dlist lst, void *element);

/**
 * Creates a new list with the same elements.
 * The data of the elements is not duplicated but now is kept in both lists simultaneously.
 */
dlist dlist_copy(dlist lst);

#endif
